/**
 * Angular2 port of our tiny decorations,
 * only the decorations are ported which make actually sense to be ported
 *
 * Fow now it makes only sense to port the rest part
 * into angular2
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.C_INJECTIONS = "__injections__";
    exports.C_REQ_PARAMS = "__request_params__";
    exports.C_PATH_VARIABLES = "__path_variables__";
    exports.C_REQ_BODY = "__request_body__";
    exports.C_REQ_META_DATA = "__request_meta__";
    var C_REST_RESOURCE = "__rest_res__";
    var C_REST_INIT = "__rest_init__";
    exports.POST_INIT = "__post_init__";
    exports.POST_INIT_EXECUTED = "__post_init__exec__";
    exports.REST_ABORT = "__REST_ABORT__";
    var C_RESTFUL = "__restful__";
    var C_UDEF = "undefined";
    exports.PARAM_TYPE = {
        URL: "URL",
        REQUEST: "REQUEST",
        BODY: "BODY"
    };
    exports.REST_TYPE = {
        POST: "POST",
        GET: "GET",
        PUT: "PUT",
        PATCH: "PATCH",
        DELETE: "DELETE"
    };
    /**
     * helper function  which determines the injector annotate function
     *
     * @returns {any|((fn:Function, strictDi?:boolean)=>string[])|((inlineAnnotatedFunction:any[])=>string[])}
     */
    var getAnnotator = function () {
        return angular.injector.$$annotate || angular.injector.annotate;
    };
    function PostConstruct() {
        return function (target, propertyName, descriptor) {
            target[exports.POST_INIT] = target[propertyName];
        };
    }
    exports.PostConstruct = PostConstruct;
    function executePostConstruct(_instance, ctor) {
        if (ctor.prototype[exports.POST_INIT] && !ctor.prototype[exports.POST_INIT_EXECUTED]) {
            ctor.prototype[exports.POST_INIT_EXECUTED] = true;
            ctor.prototype[exports.POST_INIT].apply(_instance, arguments);
        }
    }
    exports.executePostConstruct = executePostConstruct;
    function strip(inArr) {
        var retArr = [];
        if (C_UDEF == typeof inArr || null == inArr) {
            return inArr;
        }
        for (var cnt = 0, len = inArr.length; cnt < len; cnt++) {
            var element = inArr[cnt];
            if (C_UDEF != typeof element) {
                retArr.push(element);
            }
        }
        return retArr;
    }
    /**
     * extensive value mapping helper
     *
     * @param requiredKeys a set of keys which need to be processed regardless of source having it or not
     * @param source a source key value holder
     * @param target the target key value holder receiving the values
     * @param overwrite if set to true the target will be overwritten even if it exists
     * @param mappingAllowed checks whether the mapping is allowed on the current key
     * @param mapperFunc a mapper function which transforms the values according to the key
     */
    function map(requiredKeys, source, target, overwrite, mappingAllowed, mapperFunc) {
        var map = (requiredKeys || {});
        for (var key in source) {
            map[key] = 1;
        }
        for (var key in map) {
            if (!mappingAllowed || mappingAllowed(key)) {
                var mappedVal = (mapperFunc) ? mapperFunc(key) : undefined;
                var cDefMapped = C_UDEF != typeof mappedVal;
                if ((C_UDEF != typeof source[key] && overwrite) ||
                    (cDefMapped && overwrite) ||
                    (C_UDEF != typeof source[key] && (C_UDEF == typeof target[key] || null == target[key])) ||
                    (cDefMapped && (C_UDEF == typeof target[key] || null == target[key]))) {
                    var val = (cDefMapped) ? mappedVal : source[key];
                    if (C_UDEF != typeof val) {
                        target[key] = val;
                    }
                }
            }
        }
    }
    /**
     * generic create if not exist for properties,
     * used all over the system
     *
     *
     * @param target the target which receives the property
     * @param propertyKey the key
     * @param factory a factory function which produces the value of the property
     * @returns {any} whatever the factory returns or is already defined
     */
    function getOrCreate(target, propertyKey, factory) {
        if (!target[propertyKey]) {
            target[propertyKey] = factory();
        }
        return target[propertyKey];
    }
    /**
     * fetches the request metadata attached to the taerget
     *
     * @param target
     * @returns {any}
     */
    function getRequestMetaData(target, createIfNotExists) {
        if (createIfNotExists === void 0) { createIfNotExists = true; }
        return (createIfNotExists) ? getOrCreate(target, exports.C_REQ_META_DATA, function () {
            return {};
        }) : getOrCreate(target, exports.C_REQ_META_DATA, function () {
        });
    }
    function getRequestParams(target, numberOfParams) {
        var metaData = getRequestMetaData(target);
        return getOrCreate(metaData, exports.C_REQ_PARAMS, function () {
            return new Array(numberOfParams);
        });
    }
    function getPathVariables(target, numberOfParams) {
        var metaData = getRequestMetaData(target);
        return getOrCreate(metaData, exports.C_PATH_VARIABLES, function () {
            return new Array(numberOfParams);
        });
    }
    function getRequestBody(target) {
        var metaData = getRequestMetaData(target);
        if (metaData[exports.C_REQ_BODY]) {
            throw Error("Only one @RequestBody per method allowed");
        }
        return metaData[exports.C_REQ_BODY] = {};
    }
    /**
     * Extended helpers which
     * are far off from any angular spec
     *
     * TODO work in progress
     */
    var extended;
    (function (extended) {
        var $resource = null;
        /**
         * config which can be overridden by the application
         *
         * @type {{url: string; method: (REST_TYPE | any); cancellable: boolean; cache: boolean}}
         */
        extended.DefaultRestMetaData = {
            method: exports.REST_TYPE.GET,
            cancellable: true,
            cache: false
        };
        //helper to init the param meta data with the appropriate names
        var initParamMetaData = function (paramMetaData, paramNames, pos) {
            if (!paramMetaData) {
                paramMetaData = {
                    name: paramNames[pos]
                };
            }
            else if (typeof paramMetaData === 'string' || paramMetaData instanceof String) {
                paramMetaData = {
                    name: paramMetaData
                };
            }
            else if (!paramMetaData.name) {
                paramMetaData.name = paramNames[pos];
            }
            if (paramMetaData)
                paramMetaData.pos = pos;
            return paramMetaData;
        };
        function RequestParam(paramMetaData) {
            return function (target, propertyName, pos) {
                //we can use an internal function from angular for the parameter parsing
                var paramNames = getAnnotator()(target[propertyName]);
                var finalParamMetaData = initParamMetaData(paramMetaData, paramNames, pos);
                getRequestParams(target[propertyName], paramNames.length)[pos] = (finalParamMetaData) ? finalParamMetaData : {
                    name: paramNames[pos],
                    paramType: exports.PARAM_TYPE.URL,
                    pos: pos
                };
            };
        }
        extended.RequestParam = RequestParam;
        function PathVariable(paramMetaData) {
            return function (target, propertyName, pos) {
                //we can use an internal function from angular for the parameter parsing
                var paramNames = getAnnotator()(target[propertyName]);
                var finalParamMetaData = initParamMetaData(paramMetaData, paramNames, pos);
                getPathVariables(target[propertyName], paramNames.length)[pos] = (finalParamMetaData) ? finalParamMetaData : {
                    name: paramNames[pos],
                    paramType: exports.PARAM_TYPE.URL,
                    pos: pos
                };
            };
        }
        extended.PathVariable = PathVariable;
        function RequestBody() {
            return function (target, propertyName, pos) {
                //we can use an internal function from angular for the parameter parsing
                var paramNames = getAnnotator()(target[propertyName]);
                getRequestBody(target[propertyName]);
                getRequestMetaData(target[propertyName])[exports.C_REQ_BODY] = {
                    paramType: exports.PARAM_TYPE.BODY,
                    pos: pos
                };
            };
        }
        extended.RequestBody = RequestBody;
        /**
         * Restable annotation
         * allows to class system wide rest annotations
         * @param {extended.IDefaultRestMetaData} options
         * @returns {(constructor: AngularCtor<Object>) => any}
         * @constructor
         */
        function Restable(options) {
            return function (constructor) {
                if (!options) {
                    return constructor;
                }
                var cls = (_a = (function (_super) {
                        __extends(GenericModule, _super);
                        function GenericModule() {
                            return _super.apply(this, [].slice.call(arguments).slice(0, arguments.length)) || this;
                        }
                        return GenericModule;
                    }(constructor)),
                    _a.__restOptions__ = options || {},
                    _a);
                cls[C_RESTFUL] = true;
                return cls;
                var _a;
            };
        }
        extended.Restable = Restable;
        function Rest(restMetaData) {
            return function (target, propertyName, descriptor) {
                var reqMeta = getRequestMetaData(target[propertyName]);
                //the entire meta data is attached to the function/method target.propertyName
                if (typeof restMetaData === 'string' || restMetaData instanceof String) {
                    restMetaData = {
                        url: restMetaData
                    };
                }
                if (restMetaData) {
                    //we map the defaults in if they are not set
                    //map<IDefaultRestMetaData>( {}, DefaultRestMetaData, restMetaData, false);
                    map({}, restMetaData, reqMeta, true);
                }
                if (!target.constructor[C_RESTFUL]) {
                    target.constructor[C_RESTFUL] = true;
                }
            };
        }
        extended.Rest = Rest;
        function decorateRestClass(clazz) {
            var fullService = (function (_super) {
                __extends(GenericRestService, _super);
                function GenericRestService() {
                    var _this = _super.apply(this, [].slice.call(arguments).slice(1, arguments.length)) || this;
                    _this.__restOptions__ = clazz.__restOptions__;
                    //the super constructor did not have assigned a resource
                    //we use our own
                    if (!_this.$resource) {
                        _this.$resource = arguments[0];
                    }
                    //init the rest init methods
                    for (var key in clazz.prototype) {
                        var restMeta = getRequestMetaData(clazz.prototype[key], false);
                        //no rest annotation we simply dont do anything
                        if (!restMeta) {
                            continue;
                        }
                        _this[C_REST_INIT + key]();
                    }
                    executePostConstruct(_this, clazz);
                    return _this;
                }
                return GenericRestService;
            }(clazz));
            for (var key in clazz.prototype) {
                var restMeta = getRequestMetaData(clazz.prototype[key], false);
                //no rest annotation we simply dont do anything
                if (!restMeta) {
                    continue;
                }
                decorateRestFunction(fullService, key, clazz, restMeta);
            }
            //if (!(<any>fullService).$inject || (<any>fullService).$inject.indexOf("$resource") == -1) {
            //we always auto inject a resource
            //fullService.$inject = [C_RESOURCE].concat((<any>fullService).$inject || []);
            //}
            return fullService;
        }
        extended.decorateRestClass = decorateRestClass;
        /**
         * helper to decorate the rest functions with our generic calling code
         * of the resources
         *
         * @param target
         * @param key
         * @param clazz
         * @param restMeta
         */
        function decorateRestFunction(target, key, clazz, restMeta) {
            //rest annotation found
            //First super call
            //and if the call does not return a REST_ABORT return value
            //we proceed by dynamically building up our rest resource call
            target.prototype[key] = function () {
                if (clazz.prototype[key].apply(this, arguments) === exports.REST_ABORT) {
                    return;
                }
                else {
                    var paramsMap = {};
                    var pathVars = strip(restMeta[exports.C_PATH_VARIABLES]);
                    var valueCnt = 0;
                    for (var cnt = 0; pathVars && cnt < pathVars.length; cnt++) {
                        var param = pathVars[cnt];
                        var value = (cnt < arguments.length && C_UDEF != arguments[param.pos || 0]) ? arguments[param.pos || 0] :
                            ((C_UDEF != param.defaultValue) ? param.defaultValue :
                                (param.defaultValueFunc) ? param.defaultValueFunc : undefined);
                        var val_udef = C_UDEF == typeof value;
                        if (!val_udef) {
                            paramsMap[param.name] = (param.conversionFunc) ? param.conversionFunc.call(this, value) : value;
                        }
                        else if (val_udef && param.optional) {
                            continue;
                        }
                        else {
                            throw new Error("Required parameter has no value: method " + key);
                        }
                        valueCnt++;
                    }
                    var reqParams = strip(restMeta[exports.C_REQ_PARAMS]);
                    for (var cnt = 0; reqParams && cnt < reqParams.length; cnt++) {
                        var param = reqParams[cnt];
                        var value = (cnt < arguments.length && C_UDEF != arguments[param.pos || 0]) ? arguments[param.pos || 0] :
                            ((C_UDEF != param.defaultValue) ? param.defaultValue :
                                (param.defaultValueFunc) ? param.defaultValueFunc : undefined);
                        var val_udef = C_UDEF == typeof value;
                        if (!val_udef) {
                            paramsMap[param.name] = (param.conversionFunc) ? param.conversionFunc.call(this, value) : value;
                        }
                        else if (val_udef && param.optional) {
                            continue;
                        }
                        else {
                            throw new Error("Required parameter has no value: method " + key);
                        }
                        valueCnt++;
                    }
                    var body = (restMeta[exports.C_REQ_BODY]) ? arguments[restMeta[exports.C_REQ_BODY].pos || 0] : undefined;
                    if (C_UDEF != typeof body) {
                        body = restMeta[exports.C_REQ_BODY].conversionFunc ? restMeta[exports.C_REQ_BODY].conversionFunc.call(this, body) : body;
                    }
                    //TODO we need also to return a fixed promise
                    //data in -> data out for the decorator call
                    var retPromise = (C_UDEF != typeof body) ?
                        (restMeta.decorator) ? restMeta.decorator.call(this, this[C_REST_RESOURCE + key][restMeta.method || exports.REST_TYPE.GET](paramsMap, body)) : this[C_REST_RESOURCE + key][restMeta.method || exports.REST_TYPE.GET](paramsMap, body).$promise :
                        (restMeta.decorator) ? restMeta.decorator.call(this, this[C_REST_RESOURCE + key][restMeta.method || exports.REST_TYPE.GET](paramsMap, {})) : this[C_REST_RESOURCE + key][restMeta.method || exports.REST_TYPE.GET](paramsMap, {}).$promise;
                    //list but not least we transform/decorate the promise from outside if requested
                    return retPromise;
                }
            };
            /*
             * every rest resource must be registered on service
             * construction time.
             * Hence we have to define a rest init
             * which then later is called by the constructor.
             */
            target.prototype[C_REST_INIT + key] = function () {
                if (!(this.$resource)) {
                    throw Error("rest injectible must have a $resource instance variable");
                }
                var mappedParams = {};
                var paramDefaults = {};
                var pathVars = strip(restMeta[exports.C_PATH_VARIABLES]);
                var pathVariables = [];
                for (var cnt = 0; pathVars && cnt < pathVars.length; cnt++) {
                    pathVariables.push(":" + pathVars[cnt].name);
                    mappedParams[pathVars[cnt].name] = "@" + pathVars[cnt].name;
                    if (C_UDEF != typeof pathVars[cnt].defaultValue) {
                        paramDefaults[pathVars[cnt].name] = pathVars[cnt].defaultValue;
                    }
                }
                var reqParams = strip(restMeta[exports.C_REQ_PARAMS]);
                for (var cnt = 0; reqParams && cnt < reqParams.length; cnt++) {
                    var param = reqParams[cnt];
                    mappedParams[param.name] = "@" + param.name;
                    if (C_UDEF != typeof param.defaultValue) {
                        paramDefaults[param.name] = param.defaultValue;
                    }
                }
                //defaults first the local one from the outer service
                map({}, this.__restOptions__ || {}, restMeta, false);
                //and if that one does not exist the one from the default settings
                map({}, extended.DefaultRestMetaData, restMeta, false);
                var url = (this.$rootUrl || restMeta.$rootUrl || "") + restMeta.url + ((pathVariables.length) ? "/" + pathVariables.join("/") : "");
                var restActions = {};
                var method = restMeta.method || "GET";
                restActions[method] = {};
                var _t = this;
                //we apply all defaults, first the service default then the global default
                map({
                    method: 1,
                    cache: 1,
                    isArray: 1,
                    cancellable: 1,
                    requestBody: 1
                }, /*reqired mappings always returning a value*/ restMeta, /*source*/ restActions[method], /*target*/ false, /*overwrite*/ function (key) {
                    return (key != "url") && (key != "decorator");
                }, //mapping allowed?
                function (key) {
                    switch (key) {
                        case "method":
                            return method;
                        case "cache":
                            return !!restMeta.cache;
                        case "isArray":
                            return !!restMeta.isArray;
                        case "cancellable":
                            return C_UDEF == typeof restMeta.cancellable ? true : restMeta.cancellable;
                        case "transformResponse":
                            return restMeta.transformResponse ? function () {
                                var args = [];
                                for (var _i = 0; _i < arguments.length; _i++) {
                                    args[_i] = arguments[_i];
                                }
                                return restMeta.transformResponse.apply(_t, args);
                            } : undefined;
                        case "requestBody":
                            return !!restMeta[exports.C_REQ_BODY];
                        default:
                            return restMeta[key];
                    }
                });
                this[C_REST_RESOURCE + key] = this.$resource(url, paramDefaults, restActions);
            };
        }
    })(extended = exports.extended || (exports.extended = {}));
});
//# sourceMappingURL=TinyDecorations_ng.js.map